/*
** 		Sambar Server JAVA Engine Implementation
**
**		Confidential Property of Tod Sambar
**		(c) Copyright Tod Sambar 1999
**		All rights reserved.
*/
package com.sambar.javaeng;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.Enumeration;

/* 
** Access to Sambar API via native methods
*/
public class SambarAPI 
{
	static 
	{
		System.out.println("Loading javaeng library");

		try 
		{
			System.loadLibrary("sambarje");
		}
		catch(Exception ex) 
		{
			ex.printStackTrace();
		}

		System.out.println("Done loading javaeng library");
	}

	// -------------------- Internal codes --------------------
	public final static int FAIL = 0;
	public final static int SUCCEED = 1;

	// -------------------- Log Levels --------------------
	public final static int	LOG_FATAL = 10;
	public final static int	LOG_ERROR = 20;
	public final static int	LOG_WARN = 30;
	public final static int	LOG_INFO = 40;
	public final static int	LOG_TRACE = 50;

	// -------------------- Engine defines --------------------
	public final static String SESSION_ID = "JAVAENGID";

	// -------------------- Generic methods --------------------
	public static native void   	log(long req, int loglevel, String message);
	public static native void   	logctx(long javaeng, int loglevel, 
										String message);
	public static native void   	putb(long req, int val);
	public static native void   	puta(long req, byte b[], int off, int len);
	public static native void   	puts(long req, String text);
	public static native void   	flush(long req);
	public static native void   	close(long req);

	// -------------------- Dispatcher methods --------------------
	public static native void   	forward(long req, String name, 
										String path, String query);
	public static native void   	include(long req, String name, 
										String path, String query);

	// -------------------- ServletContext methods --------------------
	public static native String 	getMimeType(long javaeng, String file);
	public static native byte [] 	getInitParameter(long javaeng, String name);
	public static native String 	getRealPath(long javaeng, String path);
	public static native String 	getServerInfo(long javaeng);
	public static native String 	getServletsPath(long javaeng);
	public static native String 	getGenericServlet(long javaeng);
	public static native String 	getClassPath(long javaeng);

	// -------------------- ServletRequest methods --------------------
	public static native String 	getServletName(long req);
	public static native byte [] 	getContent(long req);
	public static native int    	getContentLength(long req);
	public static native String 	getContentType(long req);
	public static native byte [] 	getParameter(long req, String name);
	public static native void 		setParameter(long req, String name, String val);
	public static native int		getParameterCount(long req);
	public static native byte []	getParameterName(long req, int pos);
	public static native byte []	getParameterValue(long req, int pos);
	public static native String 	getProtocol(long req);
	public static native String 	getScheme(long req);
	public static native String 	getHostname(long req);
	public static native String 	getServerName(long req);
	public static native int    	getServerPort(long req);
	public static native String 	getRemoteAddr(long req);
	public static native String 	getRemoteHost(long req);
	public static native int 		isSecure(long req);

	// -------------------- HTTPServletRequest methods --------------------
	public static native String 	getAuthType(long req);
	public static native String 	getAuthPass(long req);
	public static native String 	getHeader(long req, String name);
	public static native int 		wasHeaderSent(long req);
	public static native String 	getMethod(long req);
	public static native String 	getPathInfo(long req);
	public static native String 	getPathTranslated(long req);
	public static native String 	getQueryString(long req);
	public static native String 	getRemoteUser(long req);
	public static native String 	getRequestURI(long req);
	public static native String 	getServletPath(long req);
	public static native String 	getContextPath(long req);

	// -------------------- SambarRequest methods --------------------
	public static native int 		isSambarAdmin(long req);
	public static native int 		isSambarUser(long req);
	public static native String 	tmpDirectory(long req);

	// -------------------- SambarResponse methods --------------------
	public static native void 		setKeepAlive(long req);
	public static native int 		canChunk(long req);
	public static native void 		setChunk(long req);
	public static native int 		canCompress(long req);
	public static native void 		setCompress(long req);
}
